@@ -4,9 +4,9 @@
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
 *  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-*  2009, 2010 Andrew Makhorin, Department for Applied Informatics,
-*  Moscow Aviation Institute, Moscow, Russia. All rights reserved.
-*  E-mail: <mao@gnu.org>.
+*  2009, 2010, 2011, 2013 Andrew Makhorin, Department for Applied
+*  Informatics, Moscow Aviation Institute, Moscow, Russia. All rights
+*  reserved. E-mail: <mao@gnu.org>.
 *
 *  GLPK is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
@@ -185,7 +185,7 @@ int glp_interior(glp_prob *P, const glp_iptcp *parm)
       }
 #endif
       /* scale the resultant LP */
-      {  LIBENV *env = lib_link_env();
+      {  ENV *env = get_env_ptr();
          int term_out = env->term_out;
          env->term_out = GLP_OFF;
          glp_scale_prob(prob, GLP_SF_EQ);
