@@ -3,9 +3,10 @@
 /***********************************************************************
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
-*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07, 08 Andrew Makhorin,
-*  Department for Applied Informatics, Moscow Aviation Institute,
-*  Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
+*  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+*  2009, 2010, 2011, 2013 Andrew Makhorin, Department for Applied
+*  Informatics, Moscow Aviation Institute, Moscow, Russia. All rights
+*  reserved. E-mail: <mao@gnu.org>.
 *
 *  GLPK is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@
 ***********************************************************************/
 
 #define _GLPSTD_STDIO
-#include "glplib.h"
+#include "glpenv.h"
 #include "glpmpl.h"
 
 /**********************************************************************/
@@ -301,7 +302,7 @@ SET *select_set
       if (node == NULL || avl_get_node_type(node) != A_SET)
          error(mpl, "%s not a set", name);
       set = (SET *)avl_get_node_link(node);
-      if (set->assign != NULL)
+      if (set->assign != NULL || set->gadget != NULL)
          error(mpl, "%s needs no data", name);
       set->data = 1;
       return set;
